/**
 * @file
 * VuoList_VuoNoise C type definition.
 *
 * @copyright Copyright © 2012–2014 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoNoise_H
#define VuoList_VuoNoise_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoNoise VuoList_VuoNoise
 * A list of @ref VuoNoise elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoNoise_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoNoise elements.
 */
typedef const struct VuoList_VuoNoise_struct { void *l; } * VuoList_VuoNoise;
/// @}
#endif

/**
 * Creates a new list of @ref VuoNoise elements.
 */
VuoList_VuoNoise VuoListCreate_VuoNoise(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoNoise VuoListCopy_VuoNoise(const VuoList_VuoNoise list);

/**
 * Returns the @ref VuoNoise at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoNoise VuoListGetValue_VuoNoise(const VuoList_VuoNoise list, const unsigned long index);

/**
 * Changes the @ref VuoNoise at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoNoise(const VuoList_VuoNoise list, const VuoNoise value, const unsigned long index);

/**
 * Inserts the `VuoNoise` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoNoise(const VuoList_VuoNoise list, const VuoNoise value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoNoise(VuoList_VuoNoise list, const VuoNoise value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoNoise(VuoList_VuoNoise list, const VuoNoise value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoNoise(VuoList_VuoNoise list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoNoise(VuoList_VuoNoise list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoNoise(VuoList_VuoNoise list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoNoise(VuoList_VuoNoise list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoNoise(VuoList_VuoNoise list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoNoise(VuoList_VuoNoise list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoNoise(VuoList_VuoNoise list);

/**
 * Removes the `VuoNoise` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoNoise(VuoList_VuoNoise list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoNoise(const VuoList_VuoNoise list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoNoise VuoList_VuoNoise_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoNoise_getJson(const VuoList_VuoNoise value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoNoise_getSummary(const VuoList_VuoNoise value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoNoise VuoList_VuoNoise_makeFromString(const char *str);
char * VuoList_VuoNoise_getString(const VuoList_VuoNoise value);
/// @}

/**
 * @}
 */

#endif
